/*
 *  Copyright 2010, 2011 Christopher Pheby
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.jadira.usertype.dateandtime.shared.spi;

import java.sql.Date;
import java.sql.Types;

import org.hibernate.Hibernate;
import org.hibernate.type.DateType;
import org.jadira.usertype.dateandtime.shared.reflectionutils.Hibernate36Helper;

public abstract class AbstractDateColumnMapper<T> extends AbstractColumnMapper<T, Date> {

    private static final long serialVersionUID = -8841076386862845448L;

    public final DateType getHibernateType() {
        if (Hibernate36Helper.isHibernate36ApiAvailable()) {
            return (DateType) Hibernate36Helper.getHibernateType("DATE");
        } else {
            return (DateType) Hibernate.DATE;
        }
    }

    public final int getSqlType() {
        return Types.DATE;
    }

    public abstract T fromNonNullValue(Date value);

    public abstract T fromNonNullString(String s);

    public abstract Date toNonNullValue(T value);

    public abstract String toNonNullString(T value);
}
